
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _RAD             = 0       
  _DEG             = 1
  _KM              = 2
  _MI              = 3
  _NM              = 4
  _M               = 5
  _MPS             = 6
  _KPH             = 7
  _KNOT            = 8
  _MPH             = 9 
  _MIN             = 10
  _HOUR            = 11

  
  
  LCD_PIN       = 15                                        'For AMC1602AR serial LCD on P14
  LCD_BAUD      = 19200                                    'Baud rate for communication to AMC1602AR LCD is set
  'Pin_Choke     = 0             'blue wire
  'Pin_Ign       = 1             'purple
  'Pin_Start     = 2             'orange
  'Pin_oil       = 3             'lilac  
  Pin_tacho     = 4             ' Grey wire in control box
  Pin_Flo       = 5             ' Pin for connection to th flow sensor
  SW3_Pin       = 6             ' Pin for the Increment switch
  SW4_Pin       = 7             ' Pin for the Decrement switch                        
  serRecv       = 8             ' Pin for GPS data (NB, this is not explicitly referred to in this code)
  Pin_CS        = 9             ' Pin for SD card
  Pin_DI        = 10            ' Pin for SD card
  Pin_CLK       = 11            ' Pin for SD card
  Pin_DO        = 12            ' Pin for SD card
  SW1_Pin       = 13            ' Pin for the mode increment switch
  SW2_Pin       = 14            ' Pin for the mode decrement switch
  Ign_LED       = 18
  'Start_LED     = 17
  'Choke_LED     = 18
  Neutral_LED   = 18
  'Neutral_LED2  = 20
  Warn_LED      = 22            ' Pin for the warning / indication LED on thefront of the box                       
  'Pin_CD        = 24            ' Pin for the 'card detect' signal for the SD card (NB not used)
  Speed         =  3
  
  Version       = 16
  
  comma         = 0             ' Enumerated value (whether to write a comma)     
  CR            = 1             ' Enumerated value (whether to write a carriage return)                                                          
  Selected      = 1             ' Enumerated value (switch position)
  NotSelected   = 0             ' Enumerated value (switch position)
  
  Divisor = 1_000_000
OBJ
  lcd           :               "Lcd_NHD-0420D3Z-FL-GBW"
  FS            :               "FloatString"
  num           :               "Simple_Numbers"                                        ' This object handles simple numerical operations
  eeprom        :               "Propeller Eeprom"
  gps           :               "GPS_Float_Lite"
  sd            :               "fsrw" 
VAR 
   byte Mode          
   byte ModeMax
   byte oldmode
   long maxSeen
  
   byte Neutral
   byte rpm                     ' ***
   byte Tacho[3]
   long trendRAte
   byte stack1[40]
   byte stack2[40]  
   byte stack3[40]
   long calib
   long hist[20]
   byte dataOut[32]
   byte Pax, remainder
   
   long floRate
   long floUsed
   long FloCount
   long TachCount
   long tempFlo
  
   long fuelrem
   long VMG, lat, lon
   long Rate
   long Factor
   long filename
   
pub main |r , conter, ctq, q, pwr, ftr, ct
' r     - is used to store values returned from other methods.
' ctq   - used in the calculation of the calibration constant.
' q     - used in the calculation of the calibration constant.
' pwr   - used in the calculation of the calibration constant.
' ftr   - used in the calculation of the calibration constant.


'Start the LCD display, passing the pin number and baud rate to the LCD object.
lcd.init( LCD_PIN,LCD_BAUD)                                                     ' Start the LCD display


' Set up the custom characters in the LCD display. These are used for the bargraph of fuel remaining. The process is:

' 1) Call 'lcdChar' - this sends the command bytes to the LCD, telling it we wish to define a custom character.
' 2) Send to the LCD the address of the custom character being defined. These are from 0 - 7
' 3) Send eight bytes defining the bitmap of the customer character. In this case these are all the same.
ct := 0
repeat 8
 lcdChar
 lcd.putc(ct++)
 repeat 8-ct  
   lcd.putc(%00000000)
 repeat ct
   lcd.putc(%11111111) 
 




lcd.cls                                                                         ' Clear the display
lcd.gotoxy(1,1)                                                                 ' Move the cursor
lcd.setbright(8)                                                                ' Set the brightess of the display
lcd.setcontrast(50)                                                             ' Set the contrast of the display



dira[Pin_Tacho]~                                        ' Tacho input
dira[Warn_LED]~~                                     ' Warning LED
dira[Neutral_LED]~~
dira[Ign_LED]~~          

Mode    := 5
ModeMax := 5
oldmode := 0
conter := 0
trendRate := 10
if maxSeen == 0
   maxSeen := 9
sd.mount_explicit(Pin_DO, Pin_CLK, Pin_DI, Pin_CS)
lcd.str(string("Version: "))
lcd.dec(Version)
lcd.gotoxy(2,1) 
r := sd.popen(string("calib.txt"), "r")
r := sd.pread(@Dataout,4)

 
ctq:=0
q:=0
repeat r - 1  
  q:=Dataout[ctq]-48               'Character
  pwr := (r- ctq - 2)             ' 10 to power of
  ftr:= 1
  repeat pwr
   ftr:=ftr * 10
 calib:=calib+(q* ftr)
 ctq++
sd.pclose


lcd.str(string("Calib fact: "))
lcd.dec(calib)
 
GPS.Init                                                                        ' Now that we have the information we require, start the GPS parsing
                                                                                '      object. The reference number for the pin on which GPS data
                                                                                '      is received is explicitly contained in this object. 

cognew(switch1,   @stack1)                                                       ' Start the cog dedicated to the monitoring og the switches.
cognew(Tack,   @stack2)
cognew(Flow,     @stack3)                                                       ' Start the cog dedicated to the monitoring of clicks from the fuel
                                                                                ' flow sensor.                        

lcd.gotoxy(2,1)                                                                 ' Change the cursor position.
startWrite                                                                      ' Open the file and get ready to write data to it

'cognew(TAck, @stack3)
  
  'r:= cognew(writeData, @stack4)




  
repeat                                                                          ' This is the main loops, which is repeated ad infinitum

  if oldmode <> mode
     callChangeMode
     oldmode := mode
      
  case Mode                                                                     ' Make a selection based upon the value of the variable 'Mode'
   
      1: ' MODE 0 - Flow data
         '####################         
        lcd.gotoxy(1,12)                                                        ' Move the position of the cursor.
        callGetRate(10)
        lcd.decf(Rate/Divisor, 2)                                              ' Display the number of whole litres in the rate. We have defined the
                                                                                '       calibration constant in micro litres, so we divide by 1E06
        lcd.str(string("."))                                                    ' Display the decimal point.
        lcd.decx((Rate - Rate/Divisor * Divisor)/100, 2)                      ' Display the decimal portion of the rate, limited to two significant places.
        
        
        lcd.str(string("  "))                                                   ' Display some space characters (just in case we need to write over anything
                                                                                '        written previously.
        
        
        lcd.gotoxy(2,12)
        lcd.decf(tempFlo/Divisor, 2)                                           ' Display the fuel used information. The variable 'Tempflo' is used to be                                                                                
        lcd.str(string("."))                                                    '       consistent with value used for determining the rate and so that  
        lcd.decx((tempFlo - (tempFlo/Divisor) * Divisor)/100, 4)              '       there are no changes between uses.
        lcd.str(string(" l"))


        lcd.gotoxy(3,12)                                                        ' Display the fuel remaining, in a similar way to above.
        lcd.decf(fuelRem/Divisor,2)
        lcd.str(string("."))
        lcd.decx((fuelRem - (fuelRem/Divisor)*Divisor)/100, 4)
        lcd.str(string(" l"))

                                                                                 ' Get ready to draw a bargraph of fuel remaining.
        lcd.gotoxy(4,1)                                                          ' This is a work in progress
        if fuelRem < 50000_0
            outa[Warn_LED] := 1
        if (fuelRem + tempflo) > 0
           r := (20*fuelRem) / (tempflo + fuelRem)
           if r<20
             repeat r
               lcd.putc(7)
                  
           repeat (20 - r)
                    lcd.str(string(" "))

      2:         
         callGPS1                                                               ' Mode 1 displays GPS data (Lat & long)
         waitcnt(50_000_000 + cnt)

      3:         
         callGPS2
         waitcnt(50_000_000 + cnt)                                              ' Mode 2 displays GPS information (number of satellites, fix validity and velocity)
      
      4:
        lcd.gotoxy(1, 8)                                                        ' Mode 3 allows the number of passengers to be changed / viewed.
        lcd.dec(Pax)

      5: callTrend
    
  writeData
PUB lcdChar
  'When called, this method sends to the LCD the control characters associated with
  '    the definition of custom characters. Since this is required eight times, it
  '    is defined once here and called repeatedly.
   
  lcd.putc($FE)
  lcd.putc($54)
PUB writeComma
' When called, this method writes a comma (ASCII 44) to the SD card. Named to make it
  '    easy to remember when.
  sd.pputc(44)
  
PUB writeData | counter

'When called, this method sends data to the SD card for storage. Each line calls 'WriteByte' and
'     sends the information to be written, the number of bytes to send and the type of delimiter
'     to follow it.

'Write data to disc.
  callWriteByte(num.dec(Cnt),                              32, comma)
  callWriteByte(FS.FloattoString(GPS.Float_Latitude_Deg),  11, comma)
  callWriteByte(FS.FloattoString(GPS.Float_Longitude_Deg), 12, comma)
  callWriteByte(num.dec(flocount),                         32, comma)
  callWriteByte(GPS.String_Speed_Over_Ground,               6, comma)
  callWriteByte(Pax,                                        1, CR)

  counter++                                                                     ' This tracks the number of times that data has been written.    
  
      if counter > 30                                                           ' If more than 30 lots of data have been written then the buffer
          sd.pflush                                                             '      of the SD car writer is flushed and the counter reset. This
          counter := 0                                                          '      means that we will not lose more than 30 data samples if
                                                                                '      there is an interruption in power. Rather lazily there is
                                                                                '      currently not a means for starting / stopping recording.
                                                                                 
          eeprom.VarBackup(@floCount, @floCount)                        ' Every 30 records that are written we update the EEPROM with
          eeprom.VarBackup(@fuelrem,  @fuelrem)                         '      fuel remaining, fuel used and number of passengers. In the
          eeprom.VarBackup(@Pax,      @Pax)                                     '      even of an interruption in power these values will not be 
                                                                                '      lost. A reset or an upload of new software will reset these
                                                                                '      (Another reason for using the bootloader and having the software
                                                                                '      on the SD card).
  
  !outa[Neutral_LED]                                                                              
PUB callchangeMode
 ' This method is called when the mode changes - it then updates the legend on the display
  '      to reflect the change of mode. This means that we don't have to re-write this text
  '      every time data is updated - we only change the numbers that might be changing.
  lcd.cls
case Mode
   1:
     callXYString(1,1,string("Flow rate:"))
     callXYString(2,1,string("Usage:"))
     callXYString(3,1,string("Remain:"))
   
   
  2:
     callXYString(1,1,string("Lat: "))
     callXYString(2,1,string("Lon: "))

  3:   
     callXYString(3,1,string("Sats: "))
     callXYString(3,16,string("Kts"))
     callXYString(4,1,string("Valid: "))

  4:
     callXYString(1,1,string("Pax: "))

  

pub switch1 | sw1, sw2, sw3, sw4  , em
' This deals with the switches. It runs in its own cog (parallel to all other activity).

 ' 1) It sets the switch pins to inputs.
 ' 2) It waits until all four switches are NOT SELECTED (initial conditions), then:
 ' 3) Waits until one (or more) switches is SELECTED, then:
 ' 4) Changes the Mode or other value.
 ' 5) Waits until all four switches are NOT SELECTED.

 ' 3) and 5) provide hysteris that stops any contact-bounce problems.

dira[SW1_PIN]~
dira[SW2_PIN]~
dira[SW3_PIN]~
dira[SW4_PIN]~                                                                             ' index is initially zero
repeat until ((ina[SW1_PIN] == notSelected) and (ina[SW2_PIN] == notSelected) and (ina[SW3_PIN] == notSelected) and (ina[SW4_PIN] == notSelected))    ' wait until all three buttons are confirmed as NOT SELECTED
'!outa[22]
repeat
   em := mode
   repeat until ((sw1 == selected) or (sw2 == selected)  or (sw3 == selected)  or (sw4 == selected))  ' wait for any to be on
      sw1 := ina[SW1_PIN]
      sw2 := ina[SW2_PIN]
      sw3 := ina[SW3_PIN]
      sw4 := ina[SW4_PIN]
 
   if (sw1 == selected)                                                        ' If switch 1 is selected update the flag and increment mode
     mode++
     sw1 := Selected
                         
     if mode == (ModeMax + 1)                                                           ' If we exceed the maximum number of choices roll-over back to 1       
        mode := 1
 
   if (sw2 == selected)                                                        ' If switch 2 is selected update the flag and decrement mode
     mode--
     sw2 := Selected
     'callchangemode(mode)     
     if mode == 0                                                                                     ' Roll-over to maximum
        mode := ModeMax


   if (sw3 == selected)                                                        ' If switch 1 is selected update the flag and increment mode
      
      if mode == 4 and Pax>0
        Pax--
      
      if mode == 1
        if fuelRem < Divisor
           fuelRem :=0
        else
           fuelRem := (fuelrem/Divisor) * Divisor - Divisor
        flocount := 0

      if mode == 5
         case trendRate
           100:
              trendRate := 50
           50:
              trendRate := 25
           25:
              trendRate := 10
           10:
              trendRate := 5
           5:
              trendRate := 1   
          

   if (sw4 == selected)                                                        ' If switch 2 is selected update the flag and decrement mode
     
     if mode ==4 and Pax < 6
        Pax++
     if mode == 1  
        fuelRem := (fuelrem/Divisor )*Divisor+ Divisor
        flocount := 0
    if mode == 5
         case trendRate
           1:
              trendRate := 5
           5:
              trendRate := 10
           10:
              trendRate := 20
           25:
              trendRate := 50
           50:
              trendRate := 100
          
   repeat until (sw1 == notSelected) and (sw2 == notSelected) and (sw3 == notSelected) and (sw4 == notSelected)  ' Wait for all switches to be selected off 
      sw1 := ina[SW1_PIN]
      sw2 := ina[SW2_PIN]
      sw3 := ina[SW3_PIN]
      sw4 := ina[SW4_PIN]
       waitcnt(50_000_000 + cnt)

   
   
 sw1 := 0                                                                                  ' Reset the sw1 flag
 sw2 := 0
 sw3 := 0
 sw4 := 0
PUB callXYString(a, b, c)
  ' So many times it is desired to place text at a specific location on the LCD display, this method has
  '      been included so that it can be achieved with a single call.
  

  lcd.gotoxy(a,b)
  lcd.str(c)
PUB Flow
  ' This method runs in its own cog and is responsible for tracking the number of pulses from the flow
  '     sensor. The flow sensor is common-collector and requires a resistor.

waitpne(|< Pin_Flo, |< Pin_Flo, 0)                                              ' Initial setup.

 repeat                                                                         ' Repeat ad infintum...
    waitpeq(|<Pin_Flo, |< Pin_Flo, 0)                                           ' Check the pin to which the flow sensor is connected. Wait until it is HIGH
    FloCount := FloCount + calib                                                ' Update the fuel used variable 'FloCount' with the calibration constant.
    if fuelRem > calib                                                          ' When decrementing the fuel remaining (variable 'FuelRem'),we need to 
      fuelrem := fuelrem - calib                                                '      consider the situation where it might go less than zero, catch that
    else                                                                        '      possibility and limit it to zero.
      fuelrem := 0
    !outa[Warn_LED]                                                             ' Toggling the state of the warning LED gives and indication of whether
                                                                                '      the fuel is flowing.
    waitpne(|< Pin_Flo, |< Pin_Flo, 0)                                          ' Wait until the pin has changed state again.

                                                                                ' Rinse and repeat...                                                                                
    
pub callGPS1 | fv

  ' This method displays lat and long from the NMEA GPS data received.

  lcd.gotoXY(1,5)
  fv := GPS.Float_Latitude_Deg
  callDispGPS(fv)
    
  lcd.gotoXY(2,5)
  fv := GPS.Float_Longitude_Deg
  callDispGPS(fv)  

pub callGPS2 

  ' This method displays time, satellite data and speed from the NMEA GPS data received.

  lcd.gotoxy(1,8)
  callDispGPSTime(GPS.Long_Hour,1)
  callDispGPSTime(GPS.Long_Minute,1)
  callDispGPSTime(GPS.Long_Second,0)

  lcd.gotoxy(3,7)
  lcd.dec(GPS.Long_SatID_In_View(0))
   
  lcd.gotoxy(3,12)   
  lcd.str(GPS.String_Speed_Over_Ground ) 
   
  lcd.gotoxy(4,7)
  lcd.str(GPS.Str_Fix_Quality)
PUB callDispGPS(fv)
  IF fv <> floatNaN
      lcd.Str(FS.FloatToString(fv))   
  ELSE
      lcd.Str(STRING("--.----"))
PUB callDispGPSTime(fv, h)
  lcd.str(num.decx(fv,2))
  if h== 1 
   lcd.str(string(":"))   
Pub Tack

dira[17]~~
repeat
 !outa[17]
 waitcnt(80_000_000 + cnt)
 
 { waitpne(|< Pin_tacho, |< Pin_tacho, 0)

  repeat
     waitpeq(|<Pin_tacho, |< Pin_tacho, 0)
     TachCount := TachCount + 1
     waitpne(|< Pin_tacho, |< Pin_tacho, 0)    }

PUB callWriteByte(v, ln, a)                                                                             ' Move data to the SD card by using the dataOut buffer.
  bytemove(@dataOut, v, ln)                                                
  sd.pwrite(@dataOut,ln)
  if a == comma
    sd.pputc(44)
  else
    sd. pputc(13)
       
PUB startWrite | Fi, L
  lcd.gotoxy(3,1)
  lcd.str(string("Mounted    "))                                                                        ' If we have got this far, tell the user the card has mounted.
  fileName++                                                                                            ' Increment the filename number
  eeprom.VarBackup(@fileName, @fileName)                                                            ' Update the filename in the EEPROM so that it is non volatile

  Fi := num.dec(fileName)                                                                               ' Convert this to a decimal string
  L  := StrSize(Fi)                                                                                     ' Get the size of this string.
   
  byte[Fi+L]   := "."                                                                                   ' Add ".csv" to the file name. This will mean that the
  byte[Fi+L+1] := "C"                                                                                   '      filenames will be 1.csv, 2.csv, etc.
  byte[Fi+L+2] := "S"
  byte[Fi+L+3] := "V"
  byte[Fi+L+4] := 0                                                                                     ' Zero required to terminate the string. 
  sd.popen(Fi, "a")
  lcd.gotoxy(4,1)
  sd.setdate(GPS.Long_Year, GPS.Long_Month, GPS.Long_Day, GPS.Long_Hour, GPS.Long_Minute, GPS.Long_Second)
  lcd.str(string("Opening: "))
  lcd.str(fi)

  callWriteByte(calib, 32, CR)
  
  waitcnt(80_000_000 + cnt)
PUB callTrend| ct, whole, remain, row, t, numbars
lcd.cls

'repeat trendRate
'  waitcnt(8_000_000 + cnt)
t:=0
repeat
  callGetRate(trendRate)                                                                                                                                                                                                                                                                                                                                                                                                                                   
  hist[0] := Rate  
  ct := 20
  repeat 20
    hist[ct] := hist[ct-1]
    ct--
  lcd.cls
  ct:= 20
  repeat 20
    row := 4
    
    numbars := ((hist[ct] * 32) / (maxSeen * 105/100))
    whole := numbars/8  
    remain := numbars - (8 * whole) 
    repeat whole
     lcd.gotoxy(row--, ct)
     lcd.putc(7)
    lcd.gotoxy(row, ct)
    lcd.putc(remain)
   ct--
 t++
PUB callGetRate(ti)

' trendRate or ti of 10 is 'normal'
  tempFlo := floCount                                                     ' Store the current value of the number representing fuel flow.
  repeat trendRate
   waitcnt(8_000_000 + cnt)                                               ' Wait 80 million counts (one second for the prescibed clock settings)
  Rate := (floCount - tempFlo) * 60 * 10/trendRate                                      ' Determine the change in value of the number representing fuel flow
  if RAte > maxSeen                                                       '       and multiply it by sixty to make it representative of the change
     maxSeen := RAte                                                      '       over a minute. (Some filtering / smoothing may be apropriate).
     eeprom.VarBackup(@maxSeen, @maxSeen)       
DAT

floatNaN       LONG $7FFF_FFFF                 'NaN code     